@@ -41,6 +41,9 @@ template<class MeshType>
 size_t Index(MeshType &m, const typename MeshType::EdgeType &e) {return &e-&*m.edge.begin();}
 template<class MeshType>
 size_t Index(MeshType &m, const typename MeshType::HEdgeType &h) {return &h-&*m.hedge.begin();}
+template <class MeshType>
+size_t Index(MeshType &m, const typename MeshType::TetraType &t) { return &t - &*m.tetra.begin(); }
+
 
 template<class MeshType>
 size_t Index(MeshType &m, const typename MeshType::VertexType *vp) {return vp-&*m.vert.begin();}
@@ -50,6 +53,9 @@ template<class MeshType>
 size_t Index(MeshType &m, const typename MeshType::EdgeType*  e) {return e-&*m.edge.begin();}
 template<class MeshType>
 size_t Index(MeshType &m, const typename MeshType::HEdgeType*  h) {return h-&*m.hedge.begin();}
+template <class MeshType>
+size_t Index(MeshType &m, const typename MeshType::TetraType *t) { return t - &*m.tetra.begin(); }
+
 
 template<class MeshType>
 bool IsValidPointer( MeshType & m, const typename MeshType::VertexType *vp) { return ( m.vert.size() > 0 && (vp >= &*m.vert.begin()) && (vp <= &m.vert.back()) ); }
@@ -59,6 +65,8 @@ template<class MeshType>
 bool IsValidPointer(MeshType & m, const typename MeshType::FaceType   *fp) { return ( m.face.size() > 0 && (fp >= &*m.face.begin()) && (fp <= &m.face.back())); }
 template<class MeshType>
 bool IsValidPointer(MeshType & m, const typename MeshType::HEdgeType  *hp) { return ( m.hedge.size() > 0 && (hp >= &*m.hedge.begin()) && (hp <= &m.hedge.back())); }
+template <class MeshType>
+bool IsValidPointer(MeshType &m, const typename MeshType::TetraType *tp) { return (m.tetra.size() > 0 && (tp >= &*m.tetra.begin()) && (tp <= &m.tetra.back())); }
 
 template <class MeshType, class ATTR_CONT>
 void ReorderAttribute(ATTR_CONT &c, std::vector<size_t> & newVertIndex, MeshType & /* m */){
@@ -106,8 +114,12 @@ public:
   typedef typename MeshType::HEdgeIterator HEdgeIterator;
   typedef typename MeshType::HEdgeContainer HEdgeContainer;
 
-  typedef typename MeshType::CoordType     CoordType;
+  typedef typename MeshType::TetraType TetraType;
+  typedef typename MeshType::TetraPointer TetraPointer;
+  typedef typename MeshType::TetraIterator TetraIterator;
+  typedef typename MeshType::TetraContainer TetraContainer;
 
+  typedef typename MeshType::CoordType     CoordType;
 
   typedef typename MeshType::PointerToAttribute PointerToAttribute;
   typedef typename std::set<PointerToAttribute>::iterator AttrIterator;
@@ -174,9 +186,12 @@ public:
   static VertexIterator AddVertices(MeshType &m, size_t n, PointerUpdater<VertexPointer> &pu)
   {
     VertexIterator last;
-    if(n == 0) return m.vert.end();
+    if(n == 0) 
+      return m.vert.end();
     pu.Clear();
-    if(m.vert.empty()) pu.oldBase=0;  // if the vector is empty we cannot find the last valid element
+    
+    if(m.vert.empty())
+      pu.oldBase=0;  // if the vector is empty we cannot find the last valid element
     else {
       pu.oldBase=&*m.vert.begin();
       pu.oldEnd=&m.vert.back()+1;
@@ -201,9 +216,12 @@ public:
       for (EdgeIterator ei=m.edge.begin(); ei!=m.edge.end(); ++ei)
         if(!(*ei).IsD())
         {
-          if(HasEVAdjacency (m)) { pu.Update((*ei).V(0)); pu.Update((*ei).V(1));}
+          // if(HasEVAdjacency (m)) 
+          pu.Update((*ei).V(0));
+          pu.Update((*ei).V(1));
           //							if(HasEVAdjacency(m))   pu.Update((*ei).EVp());
         }
+
       HEdgeIterator hi;
       for (hi=m.hedge.begin(); hi!=m.hedge.end(); ++hi)
         if(!(*hi).IsD())
@@ -214,6 +232,12 @@ public:
           }
         }
 
+      for (TetraIterator ti = m.tetra.begin(); ti != m.tetra.end(); ++ti)
+        if (!(*ti).IsD())
+          for (int i = 0; i < 4; ++i)
+            if ((*ti).cV(i) != 0)
+              pu.Update((*ti).V(i));
+
       // e poiche' lo spazio e' cambiato si ricalcola anche last da zero
     }
     size_t siz=(size_t)(m.vert.size()-n);
@@ -287,7 +311,6 @@ public:
             */
   static EdgeIterator AddEdges(MeshType &m, size_t n, PointerUpdater<EdgePointer> &pu)
   {
-    EdgeIterator last;
     if(n == 0) return m.edge.end();
     pu.Clear();
     if(m.edge.empty()) pu.oldBase=0;  // if the vector is empty we cannot find the last valid element
@@ -298,6 +321,9 @@ public:
 
     m.edge.resize(m.edge.size()+n);
     m.en+=int(n);
+    size_t siz=(size_t)(m.edge.size()-n);
+    EdgeIterator firstNewEdge = m.edge.begin();
+    advance(firstNewEdge,siz);
 
     typename std::set<typename MeshType::PointerToAttribute>::iterator ai;
     for(ai = m.edge_attr.begin(); ai != m.edge_attr.end(); ++ai)
@@ -314,22 +340,25 @@ public:
               if ((*fi).cFEp(i)!=0) pu.Update((*fi).FEp(i));
         }
 
-      if(HasVEAdjacency(m))
+      if(HasVEAdjacency(m)){
         for (VertexIterator vi=m.vert.begin(); vi!=m.vert.end(); ++vi)
           if(!(*vi).IsD())
             if ((*vi).cVEp()!=0) pu.Update((*vi).VEp());
-
+        for(EdgeIterator ei=m.edge.begin();ei!=firstNewEdge;++ei)
+          if(!(*ei).IsD())
+          {            
+            if ((*ei).cVEp(0)!=0) pu.Update((*ei).VEp(0));
+            if ((*ei).cVEp(1)!=0) pu.Update((*ei).VEp(1));
+          }        
+      }
+      
       if(HasHEAdjacency(m))
         for (HEdgeIterator hi=m.hedge.begin(); hi!=m.hedge.end(); ++hi)
           if(!(*hi).IsD())
             if ((*hi).cHEp()!=0) pu.Update((*hi).HEp());
     }
-    size_t siz=(size_t)(m.edge.size()-n);
-
-    last = m.edge.begin();
-    advance(last,siz);
-
-    return last;// deve restituire l'iteratore alla prima faccia aggiunta;
+    
+    return firstNewEdge;// deve restituire l'iteratore alla prima faccia aggiunta;
   }
 
   /** Function to add a single edge to the mesh. and initializing it with two VertexPointer
@@ -341,6 +370,17 @@ public:
     ei->V(1)=v1;
     return ei;
   }
+  
+  /** Function to add a single edge to the mesh. and initializing it with two indexes to the vertexes
+            */
+  static EdgeIterator AddEdge(MeshType &m, size_t v0, size_t v1)
+  {
+    assert(v0!=v1);
+    assert(v0>=0 && v0<m.vert.size());
+    assert(v1>=0 && v1<m.vert.size());
+    return AddEdge(m,&(m.vert[v0]),&(m.vert[v1]));
+  }
+  
 
   /** Function to add a face to the mesh and initializing it with the three given coords
             */
@@ -496,9 +536,9 @@ public:
   static FaceIterator AddFace(MeshType &m, size_t v0, size_t v1, size_t v2)
   {
     assert((v0!=v1) && (v1!=v2) && (v0!=v2));
-    assert(v0>=0 && v0<=m.vert.size());
-    assert(v1>=0 && v1<=m.vert.size());
-    assert(v2>=0 && v2<=m.vert.size());
+    assert(v0>=0 && v0<m.vert.size());
+    assert(v1>=0 && v1<m.vert.size());
+    assert(v2>=0 && v2<m.vert.size());
     return AddFace(m,&(m.vert[v0]),&(m.vert[v1]),&(m.vert[v2]));
   }
   /** Function to add a face to the mesh and initializing it with the three given coords
@@ -646,6 +686,196 @@ public:
     return firstNewFace;
   }
 
+ //::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
+  //:::::::::::::::::TETRAS ADDER FUNCTIONS:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
+
+  /** \brief Function to add n tetras to the mesh.
+            This is the only full featured function that is able to manage correctly
+            all the official internal pointers of the mesh (like the VT and TT adjacency relations)
+            \warning Calling this function can cause the invalidation of any not-managed TetraPointer
+            just because we resize the face vector.
+            If you have such pointers you need to update them by mean of the PointerUpdater object.
+            \sa PointerUpdater
+            \param m the mesh to be modified
+            \param n the number of elements to be added
+            \param pu  a PointerUpdater initialized so that it can be used to update pointers to tetras that could have become invalid after this adding.
+            \retval the iterator to the first element added.
+            */
+  static TetraIterator AddTetras(MeshType &m, size_t n, PointerUpdater<TetraPointer> &pu)
+  {
+    //nothing to do
+    if (n == 0)
+      return m.tetra.end();
+
+    //prepare the pointerupdater info
+    pu.Clear();
+    if (m.tetra.empty())
+      pu.oldBase = 0;
+    else
+    {
+      pu.oldBase = &*m.tetra.begin();
+      pu.oldEnd  = &m.tetra.back() + 1;
+    }
+
+    //resize the tetra list and update tetra count
+    m.tetra.resize(m.tetra.size() + n);
+    m.tn += n;
+
+    //get the old size and advance to the first new tetrahedron position
+    size_t oldSize = (size_t)(m.tetra.size() - n);
+
+    TetraIterator firstNewTetra = m.tetra.begin();
+    advance(firstNewTetra, oldSize);
+
+    //for each attribute make adapt the list size
+    typename std::set<typename MeshType::PointerToAttribute>::iterator ai;
+    for (ai = m.tetra_attr.begin(); ai != m.tetra_attr.end(); ++ai)
+      ((typename MeshType::PointerToAttribute)(*ai)).Resize(m.tetra.size());
+
+    //do the update
+    pu.newBase = &*m.tetra.begin();
+    pu.newEnd  = &m.tetra.back() + 1;
+    if (pu.NeedUpdate())
+    {
+      if (HasVTAdjacency(m))
+      {
+        for (VertexIterator vi = m.vert.begin(); vi != m.vert.end(); ++vi)
+          if (!vi->IsD())
+            pu.Update(vi->VTp());
+
+        for (TetraIterator ti = m.tetra.begin(); ti != m.tetra.end(); ++ti)
+          if (!ti->IsD())
+          {
+            pu.Update(ti->VTp(0));
+            pu.Update(ti->VTp(1));
+            pu.Update(ti->VTp(2));
+            pu.Update(ti->VTp(3));
+          } 
+      }
+
+      //do edge and face adjacency
+      if (HasTTAdjacency(m))
+        for (TetraIterator ti = m.tetra.begin(); ti != m.tetra.end(); ++ti)
+          if (!ti->IsD())
+          {
+            pu.Update(ti->TTp(0));
+            pu.Update(ti->TTp(1));
+            pu.Update(ti->TTp(2));
+            pu.Update(ti->TTp(3));
+          }
+    }
+
+    return firstNewTetra;
+  }
+
+//TODO: ADD 4 FACES then add tetra
+  /** Function to add a face to the mesh and initializing it with the three given VertexPointers
+            */
+  static TetraIterator AddTetra(MeshType &m, VertexPointer v0, VertexPointer v1, VertexPointer v2, VertexPointer v3)
+  {
+    assert(m.vert.size() > 0);
+    assert((v0 != v1) && (v0 != v2) && (v0 != v3) && (v1 != v2) && (v1 != v3) && (v2 != v3));
+    assert(v0 >= &m.vert.front() && v0 <= &m.vert.back());
+    assert(v1 >= &m.vert.front() && v1 <= &m.vert.back());
+    assert(v2 >= &m.vert.front() && v2 <= &m.vert.back());
+    assert(v3 >= &m.vert.front() && v3 <= &m.vert.back());
+
+//    AddFace(m, v0, v1, v2);
+//    AddFace(m, v0, v3, v1);
+//    AddFace(m, v0, v2, v3);
+//    AddFace(m, v1, v3, v2);
+
+    PointerUpdater<TetraPointer> pu;
+    TetraIterator ti = AddTetras(m, 1, pu);
+    ti->V(0) = v0;
+    ti->V(1) = v1;
+    ti->V(2) = v2;
+    ti->V(3) = v3;
+    return ti;
+  }
+
+  /** Function to add a face to the mesh and initializing it with three indexes
+            */
+  static TetraIterator AddTetra(MeshType &m, const size_t v0, const size_t v1, const size_t v2, const size_t v3)
+  {
+    assert(m.vert.size() > 0);
+    assert((v0 != v1) && (v0 != v2) && (v0 != v3) && (v1 != v2) && (v1 != v3) && (v2 != v3));
+    assert(v0 >= 0 && v0 < m.vert.size());
+    assert(v1 >= 0 && v1 < m.vert.size());
+    assert(v2 >= 0 && v2 < m.vert.size());
+    assert(v3 >= 0 && v3 < m.vert.size());
+
+    return AddTetra(m, &(m.vert[v0]), &(m.vert[v1]), &(m.vert[v2]), &(m.vert[v3]));
+  }
+  /** Function to add a face to the mesh and initializing it with the three given coords
+            */
+  static TetraIterator AddTetra(MeshType &m, const CoordType & p0, const CoordType & p1, const CoordType & p2, const CoordType & p3)
+  {
+    VertexIterator vi = AddVertices(m, 4);
+
+    VertexPointer v0 = &*vi++;
+    VertexPointer v1 = &*vi++;
+    VertexPointer v2 = &*vi++;
+    VertexPointer v3 = &*vi++;
+
+    v0->P() = p0;
+    v1->P() = p1;
+    v2->P() = p2;
+    v3->P() = p3;
+
+    return AddTetra(m, v0, v1, v2, v3);
+  }
+
+// //requires no duplicate vertices on faces you use
+//   static TetraIterator AddTetra(MeshType &m, const FaceType & f0, const FaceType & f1, const FaceType & f2, const FaceType & f3)
+//   {
+//     assert(m.face.size() > 0);
+//     assert((f0 != f1) && (f0 != f2) && (f0 != f3) && (f1 != f2) && (f1 != f3) && (f2 != f3));
+//     assert(f1 >= 0 && f1 < m.face.size());
+//     assert(f2 >= 0 && f2 < m.face.size());
+//     assert(f3 >= 0 && f3 < m.face.size());
+//     assert(f0 >= 0 && f0 < m.face.size());
+//     //TODO: decide if you want to address this like this
+//     //ERROR: can't use position...so..could force to have no dup verts..and use pointers or avoid this kind of thing
+//     assert(f0.V(0) == f1.V(0) && f0.V(0) == f2.V(0) &&   //v0
+//             f0.V(1) == f1.V(2) && f0.V(1) == f3.V(0) &&  //v1
+//             f0.V(2) == f2.V(1) && f0.V(2) == f3.V(2) &&  //v2
+//             f1.V(1) == f2.V(2) && f1.V(1) == f3.V(1) )   //v3
+
+//     //add a tetra...and set vertices correctly
+//     PointerUpdater<TetraPointer> pu;
+//     TetraIterator ti = AddTetras(m, 1, pu);
+//     ti->V(0) = f0.V(0);
+//     ti->V(1) = f0.V(1);
+//     ti->V(2) = f0.V(2);
+//     ti->V(3) = f1.V(1);
+
+//     return ti;
+//   }
+
+  /** \brief Function to add n faces to the mesh.
+            First wrapper, with no parameters
+            */
+  static TetraIterator AddTetras(MeshType &m, size_t n)
+  {
+    PointerUpdater<TetraPointer> pu;
+    return AddTetras(m, n, pu);
+  }
+
+  /** \brief Function to add n faces to the mesh.
+            Second Wrapper, with a vector of face pointer to be updated.
+            */
+  static TetraIterator AddTetras(MeshType &m, size_t n, std::vector<TetraPointer *> &local_vec)
+  {
+    PointerUpdater<TetraPointer> pu;
+    TetraIterator t_ret = AddTetras(m, n, pu);
+
+    typename std::vector<TetraPointer *>::iterator ti;
+    for (ti = local_vec.begin(); ti != local_vec.end(); ++ti)
+      pu.Update(**ti);
+    return t_ret;
+  }
+
   /* +++++++++++++++ Deleting  ++++++++++++++++ */
 
   /** Function to delete a face from the mesh.
@@ -693,6 +923,17 @@ public:
     --m.hn;
   }
 
+  /** Function to delete a tetra from the mesh.
+            NOTE: THIS FUNCTION ALSO UPDATE tn
+           */
+  static void DeleteTetra(MeshType &m, TetraType &t)
+  {
+    assert(&t >= &m.tetra.front() && &t <= &m.tetra.back());
+    assert(!t.IsD());
+    t.SetD();
+    --m.tn;
+  }
+
   /*
             Function to rearrange the vertex vector according to a given index permutation
             the permutation is vector such that after calling this function
@@ -729,6 +970,15 @@ public:
           }
           else m.vert [ pu.remap[i] ].VEClear();
         }
+        if (HasVTAdjacency(m))
+        {
+          if (m.vert[i].IsVTInitialized())
+          {
+            m.vert[ pu.remap[i] ].VTp() = m.vert[i].cVTp();
+            m.vert[ pu.remap[i] ].VTi() = m.vert[i].cVTi();
+          }
+          else m.vert[ pu.remap[i] ].VTClear();
+        }
       }
     }
 
@@ -758,8 +1008,17 @@ public:
           assert(pu.oldBase <= (*fi).V(i) && oldIndex < pu.remap.size());
           (*fi).V(i) = pu.newBase+pu.remap[oldIndex];
         }
-    // Loop on the edges to update the pointers EV relation
-    if(HasEVAdjacency(m))
+    // Loop on the tetras to update the pointers TV relation (vertex refs)
+    for(TetraIterator ti = m.tetra.begin(); ti != m.tetra.end(); ++ti)
+      if(!(*ti).IsD())
+        for(int i = 0; i < 4; ++i)
+        {
+          size_t oldIndex = (*ti).V(i) - pu.oldBase;
+          assert(pu.oldBase <= (*ti).V(i) && oldIndex < pu.remap.size());
+          (*ti).V(i) = pu.newBase+pu.remap[oldIndex];
+        }
+    // Loop on the edges to update the pointers EV relation (vertex refs)
+    // if(HasEVAdjacency(m))
       for(EdgeIterator ei=m.edge.begin();ei!=m.edge.end();++ei)
         if(!(*ei).IsD())
         {
@@ -768,14 +1027,14 @@ public:
         }
   }
 
-  static void CompactEveryVector( MeshType &m)
+  static void CompactEveryVector(MeshType &m)
   {
     CompactVertexVector(m);
     CompactEdgeVector(m);
     CompactFaceVector(m);
+    CompactTetraVector(m);
   }
 
-
   /*!
         \brief Compact vector of vertices removing deleted elements.
         Deleted elements are put to the end of the vector and the vector is resized. Order between elements is preserved but not their position (hence the PointerUpdater)
@@ -805,7 +1064,6 @@ public:
     assert((int)pos==m.vn);
 
     PermutateVertexVector(m, pu);
-
   }
 
   /*! \brief Wrapper without the PointerUpdater. */
@@ -863,13 +1121,22 @@ public:
             m.edge[ pu.remap[i] ].VEi(1) = m.edge[i].cVEi(1);
           }
         if(HasEEAdjacency(m))
-          if (m.edge[i].cEEp(0)!=0)
+//          if (m.edge[i].cEEp(0)!=0)
           {
             m.edge[ pu.remap[i] ].EEp(0) = m.edge[i].cEEp(0);
             m.edge[ pu.remap[i] ].EEi(0) = m.edge[i].cEEi(0);
             m.edge[ pu.remap[i] ].EEp(1) = m.edge[i].cEEp(1);
             m.edge[ pu.remap[i] ].EEi(1) = m.edge[i].cEEi(1);
           }
+        if(HasEFAdjacency(m))
+//          if (m.edge[i].cEEp(0)!=0)
+          {
+            m.edge[ pu.remap[i] ].EFp() = m.edge[i].cEFp();
+            m.edge[ pu.remap[i] ].EFi() = m.edge[i].cEFi();
+            m.edge[ pu.remap[i] ].EFp() = m.edge[i].cEFp();
+            m.edge[ pu.remap[i] ].EFi() = m.edge[i].cEFi();
+          }
+
       }
     }
 
@@ -903,6 +1170,8 @@ public:
           pu.Update((*ei).VEp(i));
         if(HasEEAdjacency(m))
           pu.Update((*ei).EEp(i));
+//        if(HasEFAdjacency(m))
+//          pu.Update((*ei).EFp());
       }
   }
 
@@ -1031,6 +1300,125 @@ public:
     CompactFaceVector(m,pu);
   }
 
+/*!
+    \brief Compact tetra vector by removing deleted elements.
+
+    Deleted elements are put to the end of the vector and the vector is resized.
+    Order between elements is preserved, but not their position (hence the PointerUpdater)
+    Immediately after calling this function the \c IsD() test during the scanning a vector, is no more necessary.
+    \warning It should not be called when some TemporaryData is active (but works correctly if attributes are present)
+    */
+  static void CompactTetraVector(MeshType & m, PointerUpdater<TetraPointer> & pu)
+  {
+    //nothing to do
+    if (size_t(m.tn) == m.tetra.size())
+      return;
+
+    //init the remap 
+    pu.remap.resize(m.tetra.size(), std::numeric_limits<size_t>::max());
+    
+    //cycle over all the tetras, pos is the last not D() position, I is the index
+    //when pos != i and !tetra[i].IsD() => we need to compact and update adj
+    size_t pos = 0;
+    for (size_t i = 0; i < m.tetra.size(); ++i)
+    {
+      if (!m.tetra[i].IsD())
+      {
+        if (pos != i)
+        {
+          //import data 
+          m.tetra[pos].ImportData(m.tetra[i]);
+          //import vertex refs
+          for (int j = 0; j < 4; ++j)
+            m.tetra[pos].V(j) = m.tetra[i].cV(j);
+          //import VT adj
+          if (HasVTAdjacency(m))
+            for (int j = 0; j < 4; ++j)
+            {
+              if (m.tetra[i].IsVTInitialized(j))
+              {
+                m.tetra[pos].VTp(j) = m.tetra[i].VTp(j);
+                m.tetra[pos].VTi(j) = m.tetra[i].VTi(j);
+              }
+              else
+                m.tetra[pos].VTClear(j);
+            }
+          //import TT adj
+          if (HasTTAdjacency(m))
+            for (int j = 0; j < 4; ++j)
+            {
+              m.tetra[pos].TTp(j) = m.tetra[i].cTTp(j);
+              m.tetra[pos].TTi(j) = m.tetra[i].cTTi(j);
+            }
+        }
+        //update remapping and advance pos
+        pu.remap[i] = pos;
+        ++pos;
+      }
+    }
+
+    assert(size_t(m.tn) == pos);
+    //reorder the optional attributes in m.tetra_attr 
+    ReorderAttribute(m.tetra_attr, pu.remap, m);
+    // resize the optional atttributes in m.tetra_attr to reflect the changes
+    ResizeAttribute(m.tetra_attr, m.tn, m);
+
+    // Loop on the tetras to correct VT and TT relations
+    pu.oldBase = &m.tetra[0];
+    pu.oldEnd = &m.tetra.back() + 1;
+
+    m.tetra.resize(m.tn);
+    pu.newBase = (m.tetra.empty()) ? 0 : &m.tetra[0];
+    pu.newEnd  = (m.tetra.empty()) ? 0 : &m.tetra.back() + 1;
+
+    TetraPointer tbase = &m.tetra[0];
+
+    //Loop on the vertices to correct VT relation (since we moved things around)
+    if (HasVTAdjacency(m))
+    {
+      for (VertexIterator vi = m.vert.begin(); vi != m.vert.end(); ++vi)
+        if (!(*vi).IsD())
+        {
+          if ((*vi).IsVTInitialized() && (*vi).VTp() != 0)
+          {
+            size_t oldIndex = (*vi).cVTp() - tbase;
+            assert(tbase <= (*vi).cVTp() && oldIndex < pu.remap.size());
+            (*vi).VTp() = tbase + pu.remap[oldIndex];
+          }
+        }
+    }
+
+    // Loop on the tetras to correct the VT and TT relations
+    for (TetraIterator ti = m.tetra.begin(); ti != m.tetra.end(); ++ti)
+      if (!(*ti).IsD())
+      {
+        //VT
+        if (HasVTAdjacency(m))
+          for (int i = 0; i < 4; ++i)
+            if ((*ti).IsVTInitialized(i) && (*ti).VTp(i) != 0)
+            {
+              size_t oldIndex = (*ti).VTp(i) - tbase;
+              assert(tbase <= (*ti).VTp(i) && oldIndex < pu.remap.size());
+              (*ti).VTp(i) = tbase + pu.remap[oldIndex];
+            }
+        //TT
+        if (HasTTAdjacency(m))
+          for (int i = 0; i < 4; ++i)
+            if ((*ti).cTTp(i) != 0)
+            {
+              size_t oldIndex = (*ti).TTp(i) - tbase;
+              assert(tbase <= (*ti).TTp(i) && oldIndex < pu.remap.size());
+              (*ti).TTp(i) = tbase + pu.remap[oldIndex];
+            }
+      }
+  }
+
+  /*! \brief Wrapper without the PointerUpdater. */
+  static void CompactTetraVector(MeshType &m)
+  {
+    PointerUpdater<TetraPointer> pu;
+    CompactTetraVector(m, pu);
+  }
 
 
 public:
@@ -1065,6 +1453,7 @@ public:
     h._sizeof = sizeof(ATTR_TYPE);
     h._padding = 0;
     h._handle =   new SimpleTempData<VertContainer,ATTR_TYPE>(m.vert);
+    h._type = typeid(ATTR_TYPE);
     m.attrn++;
     h.n_attr = m.attrn;
     std::pair < AttrIterator , bool> res =  m.vert_attr.insert(h);
@@ -1141,12 +1530,12 @@ public:
   template <class ATTR_TYPE>
   static
   void
-  ClearPerVertexAttribute( MeshType & m,typename MeshType::template PerVertexAttributeHandle<ATTR_TYPE> & h){
+  ClearPerVertexAttribute( MeshType & m,typename MeshType::template PerVertexAttributeHandle<ATTR_TYPE> & h, const  ATTR_TYPE & initVal = ATTR_TYPE()){
     typename std::set<PointerToAttribute > ::iterator i;
     for( i = m.vert_attr.begin(); i !=  m.vert_attr.end(); ++i)
       if( (*i)._handle == h._handle ){
         for(typename MeshType::VertexIterator vi = m.vert.begin(); vi != m.vert.end(); ++vi)
-          h[vi] = ATTR_TYPE();
+          h[vi] = initVal;
         return;}
     assert(0);
   }
@@ -1206,6 +1595,7 @@ public:
     h._padding = 0;
     //		h._typename = typeid(ATTR_TYPE).name();
     h._handle =  new SimpleTempData<EdgeContainer,ATTR_TYPE>(m.edge);
+    h._type = typeid(ATTR_TYPE);
     m.attrn++;
     h.n_attr = m.attrn;
     std::pair < AttrIterator , bool> res =  m.edge_attr.insert(h);
@@ -1329,6 +1719,7 @@ public:
     h._sizeof = sizeof(ATTR_TYPE);
     h._padding = 0;
     h._handle =   new SimpleTempData<FaceContainer,ATTR_TYPE>(m.face);
+    h._type = typeid(ATTR_TYPE);
     m.attrn++;
     h.n_attr = m.attrn;
     std::pair < AttrIterator , bool> res =  m.face_attr.insert(h);
@@ -1423,6 +1814,136 @@ public:
     return true;
   }
 
+  /// Per Tetra Attributes
+  template <class ATTR_TYPE>
+  static bool IsValidHandle(MeshType & m, const typename MeshType::template PerTetraAttributeHandle<ATTR_TYPE> & a)
+  {
+    if (a._handle == NULL)
+      return false;
+    for (AttrIterator i = m.tetra_attr.begin(); i != m.tetra_attr.end(); ++i)
+      if ((*i).n_attr == a.n_attr)
+        return true;
+    return false;
+  }
+
+  template <class ATTR_TYPE>
+  static typename MeshType::template PerTetraAttributeHandle<ATTR_TYPE> AddPerTetraAttribute(MeshType & m, std::string name)
+  {
+    PAIte i;
+    PointerToAttribute h;
+    h._name = name;
+    if (!name.empty())
+    {
+      i = m.tetra_attr.find(h);
+      assert(i == m.tetra_attr.end());
+    }
+
+    h._sizeof = sizeof(ATTR_TYPE);
+    h._padding = 0;
+    h._handle = new SimpleTempData<TetraContainer, ATTR_TYPE>(m.tetra);
+    h._type = typeid(ATTR_TYPE);
+    m.attrn++;
+    h.n_attr = m.attrn;
+    std::pair<AttrIterator, bool> res = m.tetra_attr.insert(h);
+    return typename MeshType::template PerTetraAttributeHandle<ATTR_TYPE>(res.first->_handle, res.first->n_attr);
+  }
+
+  template <class ATTR_TYPE>
+  static typename MeshType::template PerTetraAttributeHandle<ATTR_TYPE> AddPerTetraAttribute(MeshType &m)
+  {
+    return AddPerTetraAttribute<ATTR_TYPE>(m, std::string(""));
+  }
+
+  /*! \brief gives a handle to a per-tetra attribute with a given name and ATTR_TYPE
+      \returns a valid handle. If the name is not empty and an attribute with that name and type exists returns a handle to it.
+        Otherwise return a hanlde to a newly created.
+      */
+  template <class ATTR_TYPE>
+  static typename MeshType::template PerTetraAttributeHandle<ATTR_TYPE> GetPerTetraAttribute(MeshType &m, std::string name = std::string(""))
+  {
+    typename MeshType::template PerTetraAttributeHandle<ATTR_TYPE> h;
+    if (!name.empty())
+    {
+      h = FindPerTetraAttribute<ATTR_TYPE>(m, name);
+      if (IsValidHandle(m, h))
+        return h;
+    }
+    return AddPerTetraAttribute<ATTR_TYPE>(m, name);
+  }
+
+  template <class ATTR_TYPE>
+  static typename MeshType::template PerTetraAttributeHandle<ATTR_TYPE> FindPerTetraAttribute(MeshType &m, const std::string &name)
+  {
+    assert(!name.empty());
+    PointerToAttribute h1;
+    h1._name = name;
+    typename std::set<PointerToAttribute>::iterator i;
+
+    i = m.tetra_attr.find(h1);
+    if (i != m.tetra_attr.end())
+      if ((*i)._sizeof == sizeof(ATTR_TYPE))
+      {
+        if ((*i)._padding != 0)
+        {
+          PointerToAttribute attr = (*i); // copy the PointerToAttribute
+          m.tetra_attr.erase(i);           // remove it from the set
+          FixPaddedPerTetraAttribute<ATTR_TYPE>(m, attr);
+          std::pair<AttrIterator, bool> new_i = m.tetra_attr.insert(attr); // insert the modified PointerToAttribute
+          assert(new_i.second);
+          i = new_i.first;
+        }
+        return typename MeshType::template PerTetraAttributeHandle<ATTR_TYPE>((*i)._handle, (*i).n_attr);
+      }
+    return typename MeshType::template PerTetraAttributeHandle<ATTR_TYPE>(NULL, 0);
+  }
+
+  template <class ATTR_TYPE>
+  static void GetAllPerTetraAttribute(MeshType &m, std::vector<std::string> &all)
+  {
+    all.clear();
+    typename std::set<PointerToAttribute>::const_iterator i;
+    for (i = m.tetra_attr.begin(); i != m.tetra_attr.end(); ++i)
+      if (!(*i)._name.empty())
+      {
+        typename MeshType::template PerTetraAttributeHandle<ATTR_TYPE> hh;
+        hh = Allocator<MeshType>::template FindPerTetraAttribute<ATTR_TYPE>(m, (*i)._name);
+        if (IsValidHandle<ATTR_TYPE>(m, hh))
+          all.push_back((*i)._name);
+      }
+  }
+
+  /*! \brief If  the per-face attribute exists, delete it.
+    */
+  template <class ATTR_TYPE>
+  static void DeletePerTetraAttribute(MeshType &m, typename MeshType::template PerTetraAttributeHandle<ATTR_TYPE> &h)
+  {
+    typename std::set<PointerToAttribute>::iterator i;
+    for (i = m.tetra_attr.begin(); i != m.tetra_attr.end(); ++i)
+      if ((*i)._handle == h._handle)
+      {
+        delete ((SimpleTempData<TetraContainer, ATTR_TYPE> *)(*i)._handle);
+        m.tetra_attr.erase(i);
+        return;
+      }
+  }
+
+  // Generic DeleteAttribute.
+  // It must not crash if you try to delete a non existing attribute,
+  // because you do not have a way of asking for a handle of an attribute for which you do not know the type.
+  static bool DeletePerTetraAttribute(MeshType &m, std::string name)
+  {
+    AttrIterator i;
+    PointerToAttribute h1;
+    h1._name = name;
+    i = m.tetra_attr.find(h1);
+    if (i == m.tetra_attr.end())
+      return false;
+    delete ((SimpleTempDataBase *)(*i)._handle);
+    m.tetra_attr.erase(i);
+    return true;
+  }
+
+
   /// Per Mesh Attributes
   template <class ATTR_TYPE>
   static
@@ -1447,6 +1968,7 @@ public:
     h._sizeof = sizeof(ATTR_TYPE);
     h._padding = 0;
     h._handle =  new Attribute<ATTR_TYPE>();
+    h._type = typeid(ATTR_TYPE);
     m.attrn++;
     h.n_attr = m.attrn;
     std::pair < AttrIterator , bool> res =  m.mesh_attr.insert(h);
@@ -1613,7 +2135,36 @@ public:
     pa._padding = 0;
   }
 
+    template <class ATTR_TYPE>
+  static void FixPaddedPerTetraAttribute(MeshType &m, PointerToAttribute &pa)
+  {
 
+    // create the container of the right type
+    SimpleTempData<TetraContainer, ATTR_TYPE> *_handle = new SimpleTempData<TetraContainer, ATTR_TYPE>(m.tetra);
+
+    // copy the padded container in the new one
+    _handle->Resize(m.tetra.size());
+    for (size_t i = 0; i < m.tetra.size(); ++i)
+    {
+      ATTR_TYPE *dest = &(*_handle)[i];
+      char *ptr = (char *)(((SimpleTempDataBase *)pa._handle)->DataBegin());
+      memcpy((void *)dest,
+             (void *)&(ptr[i * pa._sizeof]), sizeof(ATTR_TYPE));
+    }
+
+    // remove the padded container
+    delete ((SimpleTempDataBase *)pa._handle);
+
+    // update the pointer to data
+    pa._sizeof = sizeof(ATTR_TYPE);
+
+    // update the pointer to data
+    pa._handle = _handle;
+
+    // zero the padding
+    pa._padding = 0;
+  }
+  
   template <class ATTR_TYPE>
   static void FixPaddedPerMeshAttribute ( MeshType & /* m */,PointerToAttribute & pa){
 
