@@ -388,7 +388,7 @@ public: static void Name(std::vector<std::string> & name){name.push_back(std::st
 template <class T> class WedgeColor4f: public WedgeColor<vcg::Color4f, T> {
 public: static void Name(std::vector<std::string> & name){name.push_back(std::string("WedgeColor4f"));T::Name(name);}
 };
-template <class T> class Color4b: public Color<vcg::Color4b, T> { public:
+template <class T> class Color4b: public Color<vcg::Color4b, T> {
 public: static void Name(std::vector<std::string> & name){name.push_back(std::string("Color4b"));T::Name(name);}
 };
 
