@@ -29,7 +29,7 @@
 #ifndef __VCG_FACE_POS
 #define __VCG_FACE_POS
 
-#include <assert.h>
+#include <cassert>
 
 namespace vcg {
 namespace face {
@@ -200,10 +200,9 @@ public:
         assert(f->V(f->Prev(z))!=v && (f->V(f->Next(z))==v || f->V((z))==v));
         FaceType *nf=f->FFp(z);
         int nz=f->FFi(z);
-        assert(nf->V(nf->Prev(nz))!=v && (nf->V(nf->Next(nz))==v || nf->V((nz))==v));
+        assert(nf->V(nf->Prev(nz))!=v && (nf->V(nf->Next(nz))==v || nf->V(nz)==v));
         f=nf;
         z=nz;
-        assert(f->V(f->Prev(z))!=v && (f->V(f->Next(z))==v || f->V(z)==v));
     }
 
     /// Changes vertex maintaining the same face and the same edge
@@ -291,40 +290,12 @@ public:
         //assert(f->FFp(z)==f); // f is border along j
     }
 
-    /// Finds the next Crease half-edge border
-    /// TODO change crease flag with something more generic (per edge)
-    void NextCrease( )
-    {
-        assert(f->V(f->Prev(z))!=v && (f->V(f->Next(z))==v || f->V(z)==v));
-        assert(IsCrease()); // f is border along j
-        // Si deve cambiare faccia intorno allo stesso vertice v
-        //finche' non si trova una faccia di bordo.
-        do
-        {
-            FlipE();
-            if (!IsCrease()) FlipF();
-        }
-        while(!IsCrease());
-
-        // L'edge j e' di bordo e deve contenere v
-        assert(IsCrease() &&( f->V(z)==v || f->V(f->Next(z))==v ));
-
-        FlipV();
-        assert(f->V(f->Prev(z))!=v && (f->V(f->Next(z))==v || f->V(z)==v));
-    }
-
     /// Checks if the half-edge is of border
     bool IsBorder()const
     {
         return face::IsBorder(*f,z);
     }
 
-    /// Checks if the half-edge is of crease
-    bool IsCrease() const
-    {
-        return f->IsCrease(z);
-    }
-
     bool IsFaux() const
     {
         return (f->IsF(z));
@@ -334,7 +305,36 @@ public:
     {
         return face::IsManifold(*f,z);
     }
+    
+    void NextEdgeS( )
+    {
+        assert(f->V(f->Prev(z))!=v && (f->V(f->Next(z))==v || f->V(z)==v));
+        assert(IsEdgeS());
+        do
+        {
+            FlipE();
+            if (!IsEdgeS()) FlipF();
+        }
+        while(!IsEdgeS());
 
+        assert(IsEdgeS() &&( f->V(z)==v || f->V(f->Next(z))==v ));
+
+        FlipV();
+        assert(f->V(f->Prev(z))!=v && (f->V(f->Next(z))==v || f->V(z)==v));
+    }
+
+    bool IsFaceS() const { return f->IsS();}
+    bool IsEdgeS() const { return f->IsFaceEdgeS(z);}
+    bool IsVertS() const { return v->IsS();}
+    
+    /*!
+     * Returns the angle (in radiant) between the two edges incident on V.
+     */
+    ScalarType AngleRad() const
+    {
+        return Angle(f->V(f->Prev(z))->cP()-v->cP(), f->V(f->Next(z))->cP()-v->cP());
+    }
+    
     /*!
      * Returns the number of vertices incident on the vertex pos is currently pointing to.
      */
